library ieee;
use ieee.std_logic_1164.all;

package const is
	constant dat_w : positive := 32;
	constant adr_w : positive := 32;
	
	constant exc_cnt : positive := 9;
	--zero is reserved
	constant exc_if_align	: Natural := 1; --	 4
	constant exc_if_access	: Natural := 2; --	 8
	constant exc_if_itlb	: Natural := 3; --	 C
	constant exc_id_opcode	: Natural := 4; --	10
	constant exc_ex_overflow: Natural := 5; --	14
	constant exc_mem_align	: Natural := 6; --	18
	constant exc_mem_access	: Natural := 7; --	1C
	constant exc_mem_dtlb	: Natural := 8; --	20

-------------------------------------------------------------------
--      Cache constans
--             tag          row    col
--	|                 | x x | x x x | 0 0 |
--         31 - 7            6-5   4-2   1-0
-------------------------------------------------------------------
	constant words_per_line: positive := 8;
	constant line_row_w: positive := 2;
	constant line_col_w: positive := 3;
	constant line_tag_w: positive := 25;
	constant line_col_b: positive := 4;
	constant line_col_e: positive := 2;
	constant line_tag_b: positive := 31;
	constant line_tag_e: positive := 7;
	constant line_row_b: positive := 6;
	constant line_row_e: positive := 5;
-------------------------------------------------------------------

-------------------------------------------------------------------
--		TLB
--	|	v. adr 	|	p. adr 	| rwx |  valid	| dirty | LRU	|
--	|	31..11	|	31..11	| 3b  |	  1b	|	1b	|	xb	| x-depends on number of rows
-------------------------------------------------------------------
	constant tlb_lines: positive := 4;
	constant offset_w: positive := 11; --10..0 must be >= than cache size
	constant vp_w: positive := adr_w - offset_w; --v.adr field
	constant pp_w: positive := adr_w - offset_w; --size_of(number of pages)
	constant tlb_status_w: positive := 5; -- v d r w x
	constant v_pos: Natural := 4;
	constant d_pos: Natural := 3;
	constant r_pos: Natural := 2;
	constant w_pos: Natural := 1;
	constant x_pos: Natural := 0;
	constant tlb_phys_e: positive := offset_w;
	constant tlb_phys_b: positive := tlb_phys_e + pp_w - 1;
	constant tlb_entry_e: positive :=3;
	constant tlb_entry_b: positive :=6;
-------------------------------------------------------------------

	constant rom_w: positive := 12;
	constant ram_w: positive := 12;
	constant mem_w: positive := 13;

	subtype ram_adr_type is std_logic_vector(ram_w-1 downto 0);
	subtype rom_adr_type is std_logic_vector(rom_w-1 downto 0);
	subtype adr_type is std_logic_vector(adr_w-1 downto 0);
	subtype dat_type is std_logic_vector(dat_w-1 downto 0);
	subtype reg_adr_type is std_logic_vector(3 downto 0);
	type line_dat_type is array (words_per_line-1 downto 0) of dat_type;
	subtype line_tag_type is std_logic_vector(line_tag_w-1 downto 0);
	subtype line_row_type is std_logic_vector(line_row_w-1 downto 0);
	subtype line_col_type is std_logic_vector(line_col_w-1 downto 0);
	subtype exc_vector_type is std_logic_vector(exc_cnt-1 downto 0); --exception cause vector
	subtype tlb_stat_type is std_logic_vector(tlb_status_w-1 downto 0);
	subtype tlb_padr_type is std_logic_vector(pp_w-1 downto 0);
	subtype tlb_vadr_type is std_logic_vector(vp_w-1 downto 0);
	subtype perf_counter_type is std_logic_vector(15 downto 0);
end package;
