library ieee;
use ieee.std_logic_1164.all;

use work.const.all;


entity cpu is
port (
	clk, clkram: in std_logic;
	reset: in std_logic
);
end entity cpu;

architecture behav of cpu is

signal mem2cache, cache2mem: line_dat_type;
signal mporta_addr, mportb_addr: adr_type;
signal mporta_rd, mportb_rd, mportb_wr: std_logic;
signal icache_fill, dcache_fill, bfull, dmiss, imiss: std_logic;

signal if_next_pc : adr_type;
signal if_ir_out : dat_type;
signal if_stall, if_nop, if_isValid: std_logic;
signal if_exc_vector: exc_vector_type;

signal id_imm, id_op1, id_op2: dat_type;
signal id_sel8, id_sel16, id_sel32 : std_logic;
signal id_isValid, id_ishigh, id_islow, id_ismem, id_ldi: std_logic;
signal id_isalu, id_isaluspec, id_isstore, id_isbranch, id_ishalt: std_logic;
signal id_op_alu: std_logic_vector(3 downto 0);
signal id_rd_addr, id_rs1_addr, id_rs2_addr: reg_adr_type;
signal id_branch_cond: std_logic_vector(1 downto 0);
signal id_pc, id_branch_adr, id_jmpAdr: adr_type;
signal id_isJmp_if: std_logic;
signal id_stall, id_nop, id_load_alu_stall: std_logic;
signal id_exc_vector: exc_vector_type;
signal id_rmPC, id_trapPC: adr_type;
signal id_itlb_ena: std_logic;

signal ex_alu_data, ex_reg2: dat_type;
signal ex_mem_isJump : std_logic;
signal ex_mem_jumpAdr : adr_type;
signal ex_sel8, ex_sel16, ex_sel32 : std_logic;
signal ex_isValid, ex_ismem, ex_ldi: std_logic;
signal ex_isalu, ex_isaluspec, ex_isstore, ex_isbranch, ex_ishalt: std_logic;
signal ex_rd_addr, ex_rs2Adr: reg_adr_type;
signal ex_branch_cond: std_logic_vector(1 downto 0);
signal ex_take_branch, ex_isbranch_if, ex_regWrEna: std_logic;
signal ex_pc, ex_branchAdr: adr_type;
signal ex_stall, ex_nop: std_logic;
signal ex_branch_misspredict: std_logic;
signal ex_exc_vector: exc_vector_type;
signal ex_itlb_ena: std_logic;
signal ex_itlb_dat, ex_itlb_adr: dat_type;

signal mem_out, mem_alu_data: dat_type;
signal mem_rd_addr: reg_adr_type;
signal mem_isValid, mem_ismem, mem_ldi: std_logic;
signal mem_isalu, mem_isaluspec, mem_isstore, mem_ishalt: std_logic;
signal mem_regWrEna: std_logic;
signal mem_branch_cond: std_logic_vector(1 downto 0);
signal mem_stall, mem_nop: std_logic;
signal mem_pc, mem_rmAddress: adr_type;
signal mem_exc_vector: exc_vector_type;

signal wb_Dat : dat_type;
signal wb_Adr : reg_adr_type;
signal wb_ena: std_logic;
signal wb_trapPC_offset, wb_rmPC, wb_rmAddress: adr_type;
signal wb_isExc, wb_trapPC_ena: std_logic;
signal wb_stall, wb_ishalt, wb_isValid: std_logic;

signal ishalted: std_logic;

begin
	if_block: entity work.if_stage(behav)
		PORT MAP(clk, reset, if_stall, if_nop,

			id_isJmp_if, id_jmpAdr,
			ex_take_branch, ex_isbranch_if, ex_branchAdr,
			id_trapPC, wb_trapPC_offset, wb_isExc,
			ex_itlb_dat, ex_itlb_adr, ex_itlb_ena,

			if_next_pc,
			if_ir_out,

			if_exc_vector,
			if_isValid,

			mporta_addr, mem2cache, imiss, icache_fill
		);

	id_block: entity work.id_stage(behav)
		PORT MAP(clk, reset, id_stall, id_nop,

			if_next_pc, if_ir_out,
			if_exc_vector,
			if_isValid,

			wb_Dat, wb_Adr, wb_ena,

			id_op_alu,
			id_sel8, id_sel16, id_sel32,
			id_isValid, id_ismem, id_ldi,
			id_isalu, id_ishigh, id_islow,
			id_isstore, id_isbranch, id_isaluspec, id_ishalt,
			id_rd_addr, id_rs1_addr, id_rs2_addr, id_imm,
			id_op1, id_op2,
			id_isJmp_if, id_jmpAdr,
			id_pc, id_branch_adr, id_branch_cond,
			id_ismem, id_isstore, id_rd_addr,
			id_load_alu_stall,
			id_exc_vector,
			wb_rmPC, wb_rmAddress, wb_isExc, wb_trapPC_ena,
			id_trapPC,
			id_itlb_ena
		);

	ex_block: entity work.ex_stage(behav)
		PORT MAP(clk, reset, ex_stall, ex_nop,
			id_op1, id_op2, id_imm, id_op_alu,
			ex_alu_data,

			id_pc, id_branch_adr, id_branch_cond,
			id_sel8, id_sel16, id_sel32,
			id_isValid, id_ismem, id_ldi,
			id_isalu, id_isaluspec, id_ishigh, id_islow,
			id_isstore, id_isbranch, id_ishalt,
			id_rd_addr, id_rs1_addr, id_rs2_addr,
			id_exc_vector, id_itlb_ena,

			ex_branch_cond,
			ex_sel8, ex_sel16, ex_sel32,
			ex_isValid, ex_ismem, ex_ldi,
			ex_isalu, ex_isaluspec, ex_isstore, ex_isbranch, ex_ishalt,
			ex_rd_addr, ex_reg2,
			ex_take_branch, ex_isbranch_if, ex_pc, ex_branchAdr,
			ex_regWrEna, --calulating this here for bypassing
			ex_rs2Adr,
			ex_itlb_dat, ex_itlb_adr, ex_itlb_ena,
			ex_rd_addr, ex_alu_data, ex_regWrEna, --bypass stuff
			wb_Adr, wb_Dat, wb_ena, --bypass stuff
			ex_branch_misspredict,
			ex_exc_vector
		);

	mem_block: entity work.mem_stage(behav)
		PORT MAP(clk, reset, mem_stall, mem_nop,
			ex_pc,
			ex_alu_data,         -- address
			ex_reg2,             -- data
			mem_out,

			ex_branch_cond,
			ex_sel8, ex_sel16, ex_sel32,
			ex_isValid, ex_ismem, ex_ldi,
			ex_isalu, ex_isaluspec, ex_isstore, ex_ishalt,
			ex_rd_addr,
			ex_regWrEna,
			ex_rs2Adr,
			ex_exc_vector,

			mem_pc,
			mem_branch_cond,
			mem_isValid, mem_ismem, mem_ldi,
			mem_isalu, mem_isaluspec, mem_isstore, mem_ishalt,
			mem_rd_addr, mem_alu_data,
			mem_regWrEna,
			mem_exc_vector,
			mem_rmAddress,
			-- MEM->MEM bypass
			mem_ismem, mem_isstore,
			mem_rd_addr, mem_out,
			-- CACHE
			mportb_addr, mem2cache, cache2mem,
		        dmiss, bfull, dcache_fill,
		        mportb_rd, mportb_wr);

	wb_block: entity work.wb_stage(behav)
		PORT MAP(
			clk, reset, wb_stall, ishalted,
			mem_pc,
			mem_branch_cond,
			mem_isValid, mem_ismem, mem_ldi,
			mem_isalu, mem_isaluspec, mem_isstore, mem_ishalt,
			mem_rd_addr, mem_alu_data,
			mem_out,
			mem_regWrEna,
			mem_exc_vector,
			mem_rmAddress,
			wb_Dat, wb_Adr, wb_ena,
			wb_trapPC_offset,
			wb_rmPC, wb_rmAddress, wb_isExc, wb_trapPC_ena,
			wb_ishalt
		);

	stall_nop_ctrl: entity work.stall_nop_control(behav)
		PORT MAP(
			clk, reset,
			id_load_alu_stall,
			ex_branch_misspredict,
		        imiss, dmiss, bfull, wb_isExc, wb_ishalt,
			if_stall, id_stall, ex_stall, mem_stall, wb_stall,
			if_nop, id_nop, ex_nop, mem_nop,
			ishalted
		);

	mporta_rd <= imiss;
	memctl: entity work.memctl(rtl)
		PORT MAP(clkram, reset,
		        mporta_addr, mporta_rd, icache_fill,
		        mportb_addr, mportb_rd, mportb_wr, dcache_fill,
		        mem2cache, cache2mem);

end architecture behav;
