library ieee;
use ieee.std_logic_1164.all;
use work.const.all;


entity ex_latch is
port (
	clk, reset, stall, nop: in std_logic;
	datin: in dat_type;
	datout: out dat_type;

	i_pc: in adr_type;
	i_branch_cond: in std_logic_vector(1 downto 0);
	i_sel8, i_sel16, i_sel32: in std_logic;
	i_isValid, i_ismem, i_isldi: in std_logic;
	i_isalu, i_isaluspec, i_isstore, i_isbranch, i_ishalt: in std_logic;
	i_rd: in reg_adr_type;
	i_reg2: in dat_type;
	i_rs2Adr: in reg_adr_type;
	i_regWrEna: in std_logic;
	i_exc_vector: in exc_vector_type;

	o_pc: out adr_type;
	o_branch_cond: out std_logic_vector(1 downto 0);
	o_sel8, o_sel16, o_sel32: out std_logic;
	o_isValid, o_ismem, o_isldi: out std_logic;
	o_isalu, o_isaluspec, o_isstore, o_isbranch, o_ishalt: out std_logic;
	o_rd: out reg_adr_type;
	o_reg2: out dat_type;
	o_rs2Adr: out reg_adr_type;
	o_regWrEna: out std_logic;
	o_exc_vector: out exc_vector_type
);
end entity ex_latch;

architecture behav of ex_latch is
begin
	EX_LATCH: process(clk, reset, stall, nop)
	begin
		if reset = '1' then
			o_isValid <= '0';
			o_ismem <= '0';
			o_isldi <= '0';
			o_isalu <= '0';
			o_isaluspec <= '0';
			o_isstore <= '0';
			o_isbranch <= '0';
			o_ishalt <= '0';
			datout <= (others => '0');
			o_reg2 <= (others => '0');
			o_rd <= (others => '0');
			o_branch_cond <= (others => '0');
			o_sel8 <= '0';
			o_sel16 <= '0';
			o_sel32 <= '0';
			o_rs2Adr <= (others => '0');
			o_regWrEna <= '0';
			o_pc <= (others => '0');
			o_exc_vector <= (others => '0');
		elsif clk = '1' and clk'event and stall = '0' then
			if nop = '1' then
				o_isValid <= '0';
				o_ismem <= '0';
				o_isldi <= '0';
				o_isalu <= '0';
				o_isaluspec <= '0';
				o_isstore <= '0';
				o_isbranch <= '0';
				o_ishalt <= '0';
				o_exc_vector <= (others => '0');
			else
				o_isValid <= i_isValid;
				o_ismem <= i_ismem;
				o_isldi <= i_isldi;
				o_isalu <= i_isalu;
				o_isaluspec <= i_isaluspec;
				o_isstore <= i_isstore;
				o_isbranch <= i_isbranch;
				o_ishalt <= i_ishalt;
				o_exc_vector <= i_exc_vector;
			end if;
			o_pc <= i_pc;
			o_branch_cond <= i_branch_cond;
			datout <= datin;
			o_rd <= i_rd;
			o_reg2 <= i_reg2;
			o_sel8 <= i_sel8;
			o_sel16 <= i_sel16;
			o_sel32 <= i_sel32;
			o_rs2Adr <= i_rs2Adr;
			o_regWrEna <= i_regWrEna;
		end if;
	end process;

end architecture behav;
