library ieee;
use ieee.std_logic_1164.all;

use work.const.all;


entity id_decoder is
port (
	--clk: in std_logic;

	ins: in dat_type; --instruction
	
	--type of operation
	alu_op: out std_logic_vector(3 downto 0);
	--operand width -- for now only for MEM operations
	sel8, sel16, sel32: out std_logic;
	--instruction type
	isMem: out std_logic;
	isLdi: out std_logic;
	isAlu: out std_logic;
	isJmp: out std_logic;
	isJmpSpec: out std_logic;
	isHigh: out std_logic;
	isLow: out std_logic;
	isStore: out std_logic;
	isBranch: out std_logic;
	isAluSpec: out std_logic; --in wb, trapPC
	isHalt: out std_logic;
	--
	rdAdr: out reg_adr_type;
	rs1Adr: out reg_adr_type;
	rs2Adr: out reg_adr_type;
	--
	imm: out dat_type;
	branch_cond: out std_logic_vector(1 downto 0);
	--
	exc: out std_logic
);
end entity id_decoder;

architecture behav of id_decoder is
	signal mem_size: std_logic_vector(1 downto 0);
	signal isAluSpectmp: std_logic;
	signal isJmptmp: std_logic;
begin
	-- MEM operations need a sum in the alu.
	alu_op(1 downto 0) <= ins(27 downto 26) when ins(28) = '0' else "00";
	alu_op(3 downto 2) <= ins(25 downto 24) when ins(30) = '1' else "00";
	isMem <= ins(28);
	isLdi <= ins(29);
	isAlu <= ins(30);
	isJmptmp <= ins(31) and not ins(26);
	isJmp <= isJmptmp;
	isJmpSpec <='1' when isJmptmp = '1' and ins(25 downto 24) = "01" else
				'0';
	isBranch <= ins(31) and ins(26);	
	isHalt <= '1' when ins(31 downto 24) = "00001111" else '0';
	isLow <= '1' when ins(27 downto 26) = "00" else '0';
	isHigh <= '1' when ins(27 downto 26) = "01" else '0';
	isStore <= ins(26) and ins(28);
	branch_cond <= ins(25 downto 24);

	mem_size <= ins(25 downto 24);
	sel8 <= '1' when mem_size = "00" else '0';
	sel16 <= '1' when mem_size = "01" else '0';
	sel32 <= '1' when mem_size = "10" else '0';

	isAluSpectmp <=	'1' when ins(31 downto 26) = "000000" and (ins(25) = '1' or ins(24)='1') else
							'0';
	isAluSpec <= isAluSpectmp;
	rdAdr <= ins(15 downto 12) when ins(30) = '1' or
	                                isAluSpectmp = '1' else
	         ins(19 downto 16) when ins(28 downto 26) = "100" or
	                                ins(29) = '1' else
	         "0000";
	rs1Adr <= ins(23 downto 20);
	rs2Adr <= ins(19 downto 16); --this should stay like this because of LDI bypasses - RS2 is same as RD
	imm <= X"0000" & ins(15 downto 0) when ins(15) = '0' else
	       X"FFFF" & ins(15 downto 0);

	opcode_exc: entity work.id_opcode_exc(behav)
		PORT MAP(
			ins,
			exc
		);
end architecture behav;
