
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

use work.const.all;

entity memctl is
port(clk, reset: in std_logic;
	adr_p: in adr_type;
	rd_p: in std_logic;
	fill1: out std_logic;

	adr: in adr_type;
	rd, wr: in std_logic;
	fill2: out std_logic;

	oline: out line_dat_type;
	iline: in line_dat_type
);
end entity memctl;

architecture rtl of memctl is
	signal req, req1, req2: std_logic;
	signal memadr: adr_type;
	signal seladr: std_logic_vector(adr_w-line_col_w-3 downto 0);
	signal memrd, memwr: std_logic;
	signal mem_odat, mem_idat: dat_type;
	signal cnt, loadcnt: std_logic_vector(line_col_w downto 0);
	signal sel: std_logic_vector(words_per_line-1 downto 0);
	signal tmp_line: line_dat_type;
	signal icnt: Natural;
	signal ena_rd, fill, i_fill1, i_fill2: std_logic;
begin
	req1 <= rd_p;
	req2 <= (rd or wr) and not req1;
	req <= rd or wr or rd_p;
	i_fill1 <= req1 and cnt(line_col_w);
	i_fill2 <= req2 and cnt(line_col_w);
	fill1 <= i_fill1;
	fill2 <= i_fill2;
	fill <= i_fill1 or i_fill2;

	prio: process(req1, req2, rd_p, adr_p, rd, wr, adr, fill)
	begin
		if req1 = '1' then
			memrd <= rd_p and not fill;
			memwr <= '0';
			seladr <= adr_p(adr_w-1 downto line_col_w+2);
		elsif req2 = '1' then
			memrd <= rd and not fill;
			memwr <= wr and not fill;
			seladr <= adr(adr_w-1 downto line_col_w+2);
		else
			memrd <= '0';
			memwr <= '0';
			seladr <= (others => 'X');
		end if;
	end process;

	defload: process(req, cnt)
	begin
		if req = '0' then
			loadcnt <= (others => '0');
		elsif cnt /= words_per_line then
			loadcnt <= cnt + 1;
		else
			loadcnt <= (others => '0');
		end if;
	end process;

	defcnt: process(clk, reset)
	begin
		if reset = '1' then
			cnt <= (others => '0');
		elsif clk = '1' and clk'event then
			cnt <= loadcnt;
		end if;
	end process;

	icnt <= conv_integer(cnt(line_col_w-1 downto 0));
	oline <= tmp_line;
	ena_rd <= not fill and (rd_p or rd);

	defgen: for i in 0 to words_per_line-1 generate
		sel(i) <= '1' when icnt = i and ena_rd = '1' else '0';

		defolin: process(clk, reset, sel(i))
		begin
			if reset = '1' then
				tmp_line(i) <= (others => '0');
			elsif clk = '1' and clk'event and sel(i) = '1' then
				tmp_line(i) <= mem_odat;
			end if;
		end process;
	end generate;

	mem_idat <= iline(icnt);
	memadr <= seladr & cnt(line_col_w-1 downto 0) & "00";
        sysmem: entity work.memory(architec)
                PORT MAP(clk, reset, req,
		        memrd, memwr, 
		        memadr, mem_idat, mem_odat);
end architecture rtl;
