
RAM:	EQU	01800H
VSTKP:	EQU	7
PSTKP:	EQU	2
PAGESIZ:EQU	11
STACK:	EQU	3FFCH

ERRPC:	EQU	1
ERRITLB:EQU	2
ERROP:	EQU	4
ERROF:	EQU	5
ERRMEM:	EQU	6
ERRDTLB:EQU	7

BOOT:	LDI	STACK,R15	; INITIALIZE STACK
	LDI	ETBL,R1
	MOV	R1,I		; SET TRAP PC

	LDI	VSTKP,R4
	LDI	PAGESIZ,R2
	SLL	R4,R2,R4	; R4 = VADDR

	LDI	PSTKP,R1
	LDI	PAGESIZ,R2
	SLL	R1,R2,R1
	LDI	1EH,R2		; R=1, W=1, TLB=3
	OR	R1,R2,R1	; R1 = PHADDR | TLB | RWX
	DTLB	R1,R4

	LDI	1,R1
	ST	R1,-4(R15)	; iTLB COUNTER
	ST	R1,-8(R15)	; dTLB COUNTER

	JMP	MAIN

ETBL:	NOP
	JMP	E1
	JMP	E2
	JMP	E3
	JMP	E4
	JMP	E5
	JMP	E6
	JMP	E7
	JMP	E8

E1:	LDI	ERRPC,R1	; BAD ALIGMENT OF PC
	JMP	DIE

E2:	LDI	ERRITLB,R1	; BAD iTLB ACCESS
	JMP	DIE

E3:	ST	R1,-12(R15)	; ITLB EXCEPTION
	ST	R2,-16(R15)
	ST	R3,-20(R15)
	ST	R4,-24(R15)

	MOV	FA,R1		; R1 = VIRTUAL ADDRESS
	LDI	PAGESIZ,R2
	SRL	R1,R2,R3	; R3 = VIRTUAL PAGE
	LDI	2,R2
	SLL	R3,R2,R3	; R3 = OFFSET OF VIRTUAL PAGE IN VPAGES
	LDI	VPAGES,R4
	ADD	R3,R4,R3	; R3 = ADDRESS OF VPAGES[VPAGE]
	LD	0(R3),R3	; R3 = PHYSICAL ADDRESS | RWX

	LD	-4(R15),R4
	NOT	R4,R4
	LDI	3,R2
	AND	R4,R2,R4	; R4 = iTLB ENTRY
	ST	R4,-4(R15)
	SLL R4,R2,R4
	OR	R3,R4,R3	; R3 = PHADDR | TLB | RWX

	ITLB	R3,R1
	LD	-24(R15),R4
	LD	-20(R15),R3
	LD	-16(R15),R2
	LD	-12(R15),R1
	IRET

E4:	LDI	ERROP,R1	; BAD OPCODE
	JMP	DIE

E5:	LDI	ERROF,R1	; OVERFLOW
	JMP	DIE

E6:	LDI	ERRMEM,R1	; BAD MEMORY ALIGN
	JMP	DIE

E7:	LDI	ERRDTLB,R1	; BAD dTLB ACCESS
	JMP	DIE

E8:	ST	R1,-12(R15)	; DTLB MISS EXCEPTION
	ST	R2,-16(R15)
	ST	R3,-20(R15)
	ST	R4,-24(R15)

	MOV	FA,R1		; R1 = VIRTUAL ADDRESS
	LDI	PAGESIZ,R2
	SRL	R1,R2,R3	; R3 = VIRTUAL PAGE
	LDI	2,R2
	SLL	R3,R2,R3	; R3 = OFFSET OF VIRTUAL PAGE IN VPAGES
	LDI	VPAGES,R4
	ADD	R3,R4,R3	; R3 = ADDRESS OF VPAGES[VPAGE]
	LD	0(R3),R3	; R3 = PHYSICAL ADDRESS | RWX

	LD	-8(R15),R4
	NOT	R4,R4
	LDI	3,R2
	AND	R4,R2,R4	; R4 = dTLB ENTRY
	ST	R4,-8(R15)
	SLL R4,R2,R4
	OR	R3,R4,R3	; R3 = PHADDR | TLB | RWX

	DTLB	R3,R1
	LD	-24(R15),R4
	LD	-20(R15),R3
	LD	-16(R15),R2
	LD	-12(R15),R1
	IRET


DIE:	ST	R1,0(R15)
	HALT

VPAGES:	DATA	0005H	;page = 0, rwx = 101
	DATA	0001H	;page = 0, rwx = 001
	DATA	0801H	;page = 1, rwx = 001
	DATA	1806H	;page = 3, rwx = 110
	DATA	0806H	;page = 1, rwx = 110
	DATA	1006H	;page = 2, rwx = 110
	DATA	2006H	;page = 4; rwx = 110
	DATA	1006H	;page = 2, rwx = 110


